@@ -20,6 +20,105 @@ plt.gray()
 
 class MicroVideo: 
     '''
+    A class holding video data and various methods to edit the frames. 
+
+    Attributes
+    ----------
+    filename:str
+        The file name opened (blank if not opened in object)
+    pixelSize:float
+        The pixel size in the image, should be initialised when opening data assuming the pixel size is included, if not can easily be altered
+    pixelUnit:str
+        The unit of the pixel size
+    metadata_tags:dict
+        The metadata of the file should be stored in this dictionary (works with digital micrograph files)
+    shape:tuple
+        The shape of the video (nframes, y, x) 
+    x:int
+        The size of the x axis in the image
+    y:int    
+        The size of the y axis in the image
+    frames:numpy array
+        The video data
+
+
+    List of functions in MicroVideo
+    -------------------------------
+    Imports:
+        open_dm - Opening digital micrograph files
+
+        open_video - Opening a .mp4 or .avi video files
+
+        open_array - Loading a numpy array
+
+    Saving:
+        save_tif_stack
+
+        save_tif_sequence
+
+        write_video - save .mp4 or .avi version of the video
+
+        write_image - save video frame or video average as an image
+
+    Basic functions:
+        bin - reduce size of xy axis by binning pixels, factor is specified in call
+
+        convert_to_8bit - converts to 8bit video by scaling pixel values between 0-255.
+
+        make_scalebar - creates suitably sized scalebar
+
+        Average_frames - averages frames in groups of n
+
+        Running_average - performs a running average of the video
+
+        reset_xy - reset the object x, y and shape attributes upon change of video, useful if video is cropped.
+
+    Contrast enhancement:
+        clip_contrast - enhances contrast by making a percentage of values saturated (absolute black/white) and scaling the rest of the pixels between these (my preferred contrast enhancement)
+
+        enhance_contrast - enhances contrast based on alpha (contrast), beta (brightness) and gamma (non-linear contrast) values
+
+        eqHist - equalises histogram, ensuring even converage of pixel values from 0-255
+
+        Normalise_video - normalises contrast between frames of the video
+
+    Metadata (currently relies on dm3/dm4 metadata):
+        show_metadata - shows all metadata tags and values
+
+        get_mag - prints and returns magnification (indicated and actual)
+
+        get_voltage - prints and returns voltage
+
+        get_exposure - prints and returns frame rate, exposure time
+
+        get_date_time - prints aquisition date and time
+
+    Image filters:
+        gaussian_filter
+
+        median_filter
+
+        low_pass_filter
+
+        weiner_filter
+
+        non_local_means_filter
+
+    Plotting:
+        imshow - plots still image (either single frame or average) of video
+
+        plot_histogram - plots the histogram of the video
+
+        show_video - shows video in a jupyter notebook (very slow)
+
+    Other functions:
+        Sidebyside - Sticks two videos together so they play side by side.
+
+        motioncorrect_vid - Uses motioncor2 to align the video frames (requires motioncor exectutable to be set)
+
+        
+
+
     '''
     def __init__(self):
         self.filename = ''
@@ -106,7 +205,9 @@ class MicroVideo:
 
         self.pixelSize = float(str(voxel_size).split(',')[0].strip('('))
         self.frames = mrc.data
+        self.frames.setflags(write=1)
         self.frames = np.flip(self.frames, axis=1)
+
         self.x = self.frames.shape[1]
         self.y = self.frames.shape[0]
         self.pixelUnit='nm'
@@ -291,7 +392,7 @@ class MicroVideo:
         clip.to_videofile(name, fps)
     
 
-    def write_video(self, name=None, outdir=None, **kwargs):
+    def write_video(self, name=None, outdir=None, fps=None):
         '''
         This allows saving as an mp4 or a raw avi file (imageJ compatible)
 
@@ -303,6 +404,8 @@ class MicroVideo:
             outdir:str
                 The output directory for the files, if this directory doesn't exist (in the cwd) a new one will be created.
 
+            fps:int
+                Optionally the output movie frame rate can be added (in frames per second)
         Outputs
         -------
 
@@ -326,10 +429,13 @@ class MicroVideo:
                 os.mkdir(outdir)
             name = outdir+'/'+name
 
-        if 'fps' in kwargs:
-            fps= kwargs['fps']
-        else:
-            fps = 10    
+        
+            
+        
+        if not fps and hasattr(self,'fps'):
+            fps= self.fps    
+        elif not fps:
+            fps=10
 
         outvid = []
         for frame in self.frames:
@@ -780,6 +886,67 @@ class MicroVideo:
 
 
     '''
+
+    def change_scale_unit(self, new_unit, scaling_factor=None):
+        '''
+        Function to change the unit of the scale in the image. The method is built to allow easy transfer between nanometers ('nm') and microns ('µm'), however can be used to give any new unit providing a scaling factor to multiply the current value by is included. 
+
+        Parameters
+        ----------
+
+            new_unit : str
+                The new unit for the scale, commonly 'nm' or 'µm'
+
+            scaling_factor: int/float
+                The value to multiply the current value by to give the new scale. This isnt necessary with 'µm'/'nm' transitions, but without it any other change will change unit only.
+
+        '''
+        if new_unit==self.pixelUnit:
+            
+            print('The unit is already {}! Skipping file.'.format(new_unit))
+        elif new_unit=='nm' and self.pixelUnit=='µm':
+            self.pixelSize*=1000
+            self.pixelUnit='nm'
+        elif new_unit!='nm' and new_unit!='µm':
+            self.pixelUnit=new_unit
+
+            if scaling_factor !=None:
+                self.pixelSize*=scaling_factor
+            else:
+                print('Setting new scale unit but not changing the value, please include a scaling factor to also change the value.')
+        elif new_unit=='µm' and self.pixelUnit=='nm':
+            self.pixelSize= self.pixelSize/1000
+            self.pixelUnit='µm'
+
+        elif self.pixelUnit not in ['µm', 'nm'] and new_unit in ['µm','nm'] and scaling_factor==None:
+            print("Error! You want to switch to {} but the transition from {} is not naturally supported, please include a scaling factor (even if its just 1) to ensure correct conversion.".format(new_unit,self.pixelUnit))
+        elif self.pixelUnit not in ['µm', 'nm'] and new_unit in ['µm','nm'] and scaling_factor!=None:
+            self.pixelSize*=scaling_factor
+            self.pixelUnit=new_unit
+
+        else:
+            print('Not sure how we got here! Check inputs and try again - if genuine error, raise an issue on github!')
+
+    def set_scale(self, pixels, dist, unit):
+        '''
+        Set the scale in the image with a measurement (number of pixels and size, with unit)
+        
+        Parameters
+        ----------
+        
+            pixels : int/float
+                Number of pixels measured
+
+            dist : int/float
+                Distance measured
+            unit : str
+                Unit of measurement
+
+        '''
+        self.pixelSize=dist/pixels
+        self.pixelUnit=unit
+
+
     def choose_scalebar_size(self):
         '''
         Function for choosing scalebar size, called through make_scalebar(), not a standalone function.
@@ -1238,7 +1405,7 @@ class MicroVideo:
                 plt.hist(self.frames.ravel(), 100)
         plt.show()
 
-    def show_video(self, fps=None, loop=True,reduce_size=1, vmax=None, vmin=None):
+    def show_video(self, width=500, fps=None, loop=0):
         '''
         Method to show the video within a jupyter notebook. 
         
@@ -1248,48 +1415,27 @@ class MicroVideo:
             fps: int
                 The frame rate of the video show (in frames per second), default is to take it from the metadata, and failing that show it at 10fps. 
 
-            vmax:int or float
-                The maximum value in the image such that any value above vmax is white. 
+            width:int  
+                The width of the video shown  
             
-            vmin: int or float
-                The minimum value in the image such that any value below vmin is black.
-            
-            reduce size: int
-                Factor to reduce size when plotting the video, this is important to add for large videos to reduce the time to plot the video. 
-
+            loop:int
+                Set to 1 to make the video automatically loop. 
+    
 
 
         '''
-
-        from IPython.display import HTML
         if not fps and hasattr(self, 'fps'):
             fps=self.fps
         elif not fps and not hasattr(self, 'fps'):
             fps=10
 
-        if not vmax:
-            vmax = np.max(self.frames)
-        if not vmin:
-            vmin = np.min(self.frames)
-
-        figsize = tuple(x/(100*reduce_size) for x in self.frames.shape[1:])
-        print(figsize)
-        fig, ax = plt.subplots(figsize=figsize)
-        
-
-        ax.set_xticks([])
-        ax.set_yticks([])
-        frames = []
-        fig.tight_layout()
-        def animate(frame_number):
-            ax.imshow(self.frames[frame_number], vmax=vmax, vmin=vmin)
-            if frame_number%5==0:
-                print(str(frame_number) + '  Done!')
-            return ax
-
-        ani = animation.FuncAnimation(fig,animate,interval=1000/fps, repeat_delay=2, blit=False,repeat=loop, frames=len(self.frames), cache_frame_data=False)
-
-        return HTML(ani.to_html5_video())
+        outvid = [] 
+        for frame in self.frames:
+            frame=cv.cvtColor(frame, cv.COLOR_GRAY2BGR)
+            outvid.append(frame)
+        outvid = np.array(outvid)
+        clip = ImageSequenceClip(list(outvid), fps=fps)
+        return clip.ipython_display(width=width, loop=loop)
 
     '''------------------------------------------
     SECTION: VIDEO SPECIFIC METHODS
@@ -1429,10 +1575,75 @@ class MicroVideo:
         return MC_vid
 
 
-    
+    def display_fft(self, average=True, ret=False,vmax=None, vmin=None):
+        '''
+        Function to see the 2D Fourier transform of the video. 
+
+        Parameters
+        ----------
+
+            Average:bool 
+                Do you want to see a powerspectrum for the average of the video? True or False (default=True) if true, it will be displayed automatically unless the `ret` paraameter is used to return it as a new micrograph object. 
 
 
+            vmax: int/float
+                The 'white value' when plotting the power spectrum, such that anything above this value is set to white.
+            
+            vmin: int/float
+                The 'black value' when plotting the power spectrum, such that anything below this value is set to black.
+            
+            ret: bool 
+                Set to true if you want to return a copy of the fft as a micrograph object. Only works if average=True, if average=False, a new object will always be returned.
 
+        Returns
+        -------
+
+            If ret==True and average=True, a new micrograph object is returned with the power spectrum of the average of the video as the micrograph.image attribute, from here the same functions can be used 
+            
+            if Average==False, a new MicroVideo object is returned with the powerspectrum of each frame sa the .frames attribute. 
+
+
+        '''
+        if average:
+            av = np.sum(self.frames, axis=0)
+            fft = np.fft.fft2(av)
+            fshift=np.fft.fftshift(fft)
+            magnitude_spectrum=np.log(np.abs(fshift))
+            magnitude_spectrum= magnitude_spectrum.astype(np.float32)
+            
+            if not vmax:
+                vmax = magnitude_spectrum.max()
+            if not vmin:
+                vmin = magnitude_spectrum.min()
+
+            if ret:
+             
+                fft_ob = Micrograph()
+                fft_ob.image = magnitude_spectrum
+                fft_ob.reset_xy()
+                filename = self.filename.split('.')[:-1]
+                filename.append('_FFT')
+                fft_ob.filename = filename
+                return fft_ob
+            else:
+                plt.figure(figsize=(20,20))
+                plt.imshow(magnitude_spectrum, vmin=vmin, vmax=vmax)
+                plt.show()
+        else:
+            fft_frames = [] 
+            for frame in self.frames:
+                fft=np.fft.fft2(frame)
+                fshift=np.fft.fftshift(fft)
+                magnitude_spectrum=np.log(np.abs(fshift))
+                magnitude_spectrum=magnitude_spectrum.astype(np.float32)
+                fft_frames.append(magnitude_spectrum)
+            fft_ob = MicroVideo()
+            fft_ob.frames = np.array(fft_frames)
+            fft_ob.reset_xy()
+            filename = self.filename.split('.')[:-1]
+            filename.append('_FFT')
+            fft_ob.filename = filename
+            return fft_ob
 # I had to move default pipeline outside of the class because the filters make a new instance of the class and I didnt want to multiply the number of instances in memory. 
 # Use: default_pipeline(micrograph)
 def default_video_pipeline(filename, output_type,medfilter=0, gaussfilter=3, scalebar=True,  xybin=2, color='Auto',Average_frames=2, **kwargs):
@@ -1474,7 +1685,7 @@ def default_video_pipeline(filename, output_type,medfilter=0, gaussfilter=3, sca
     if 'name' in kwargs:
         name=kwargs['name']
     else:
-        name = '.'.join(MicroVideo_object.filename.split('.')[:1])
+        name = '.'.join(MicroVideo_object.filename.split('.')[:-1])
 
     if output_type=='Save MotionCorrected Average':
         MCor_vid = MicroVideo_object.motioncorrect_vid()
@@ -1487,6 +1698,7 @@ def default_video_pipeline(filename, output_type,medfilter=0, gaussfilter=3, sca
             aved = aved.make_scalebar()
         if xybin!=0 and xybin!=1:
             aved = aved.bin(xybin)
+        MicroVideo_object = MicroVideo_object.clip_contrast()
         if 'outdir' in kwargs:
             aved.write_image(name, outdir=kwargs['outdir']+'/Images')
         else:
@@ -1520,7 +1732,7 @@ def default_video_pipeline(filename, output_type,medfilter=0, gaussfilter=3, sca
             MicroVideo_object.convert_to_8bit()
             MicroVideo_object = MicroVideo_object.clip_contrast()
             if scalebar==True:
-                MicroVideo_object.make_scalebar(texton=texton, color=color)
+                MicroVideo_object=MicroVideo_object.make_scalebar(texton=texton, color=color)
             name = name+output_type[-4:]
             if 'outdir' in kwargs:
                 MicroVideo_object.write_video(name=name,outdir=kwargs['outdir']+'/Videos')
@@ -1529,7 +1741,9 @@ def default_video_pipeline(filename, output_type,medfilter=0, gaussfilter=3, sca
                 
         elif output_type=='Save Tif Stack':
             if scalebar==True:
-                MicroVideo_object.make_scalebar(texton=texton, color=color)
+                MicroVideo_object=MicroVideo_object.make_scalebar(texton=texton, color=color)
+                #MicroVideo_object = MicroVideo_object.clip_contrast()
+
             if 'outdir' in kwargs:
                 MicroVideo_object.save_tif_stack(name=name, outdir=kwargs['outdir']+'/Videos')
             else: 
@@ -1537,7 +1751,8 @@ def default_video_pipeline(filename, output_type,medfilter=0, gaussfilter=3, sca
 
         elif output_type=='Save Tif Sequence':
             if scalebar==True:
-                MicroVideo_object.make_scalebar(texton=texton, color=color)
+                MicroVideo_object= MicroVideo_object.make_scalebar(texton=texton, color=color)
+                #MicroVideo_object = MicroVideo_object.clip_contrast()
             if 'outdir' in kwargs:
                 MicroVideo_object.save_tif_sequence(name=name, outdir=kwargs['outdir']+'/Videos')
             else: 
