@@ -39,6 +39,73 @@ class Micrograph:
         The size of the x axis in the image
     y:int    
         The size of the y axis in the image
+    image: np array
+        The image data
+
+
+    List of Functions
+    -----------------
+    Imports:
+        open_dm - Opening digital micrograph files
+
+        open_mrc - Opening an mrc file
+
+    Saving:
+        write_image - save video frame or video average as an image
+
+    Basic functions:
+        reset_xy - reset the object x, y and shape attributes upon change of video, useful if video is cropped.
+
+        bin - reduce size of xy axis by binning pixels, factor is specified in call
+
+        convert_to_8bit - converts to 8bit video by scaling pixel values between 0-255.
+
+        make_scalebar - creates suitably sized scalebar
+
+        Average_frames - averages frames in groups of n
+
+        Running_average - performs a running average of the video
+
+    Contrast enhancement:
+        clip_contrast - enhances contrast by making a percentage of values saturated (absolute black/white) and scaling the rest of the pixels between these (my preferred contrast enhancement)
+
+        enhance_contrast - enhances contrast based on alpha (contrast), beta (brightness) and gamma (non-linear contrast) values
+
+        eqHist - equalises histogram, ensuring even converage of pixel values from 0-255
+
+        Local_normalisation - Evens out contrast across the image
+
+    Metadata (currently relies on dm3/dm4 metadata):
+        show_metadata - shows all metadata tags and values
+
+        get_mag - prints and returns magnification (indicated and actual)
+
+        get_voltage - prints and returns voltage
+
+        get_exposure - prints and returns frame rate, exposure time
+
+        get_date_time - prints aquisition date and time
+
+    Image filters:
+        gaussian_filter
+
+        median_filter
+
+        low_pass_filter
+
+        weiner_filter
+
+        non_local_means_filter
+
+    Plotting:
+        imshow - plots image
+
+        plot_histogram - plots the histogram of the image
+
+        show_pair - plots two images side by side
+
+    Others:
+        display_fft - get the 2D fourier transform of the image.
 
 
     '''
@@ -96,10 +163,10 @@ class Micrograph:
         
         #self.image = np.flip(self.image, axis=0)
         self.x = self.image.shape[1]
+        pixelUnit = dm_input['pixelUnit'][-1]
         self.y = self.image.shape[0]
         pixelSize=dm_input['pixelSize'][-1]
         #print(pixelSize,type(pixelSize))
-        pixelUnit = dm_input['pixelUnit'][-1]
         self.filename = file
         self.pixelSize= float(pixelSize)
         self.pixelUnit = pixelUnit
@@ -130,6 +197,7 @@ class Micrograph:
         voxel_size = mrc.voxel_size
         self.pixelSize = float(str(voxel_size).split(',')[0].strip('('))
         self.image = mrc.data
+        self.image.setflags(write=1)
         self.x = self.image.shape[1]
         self.y = self.image.shape[0]
         self.pixelUnit='nm'
@@ -143,9 +211,85 @@ class Micrograph:
 
     ##write a hyperspy opening function 
 
-
-
+    def open_image(self, filename, pixelSize=None, pixelUnit=None):
+        '''
+        Open a jpg, png or tif file into the micrograph object. If the tif has any metadata tags, these should be saved into the metadata_tags and will then be accessible with .show_metadata()
+        The pixel size is likely to be included somewhere within the metadata and may be loaded, but the name of the tags are not always constant, and so this may need to be searched in  the .show_metadata output.
         
+        Parameters
+        ----------
+
+            filename: str
+                Name of image file to load into the object
+            pixelSize: float
+                The pixel size in the image, not necessary but can be included here, can also be loaded with set_scale()
+            pixelUnit: str
+                The unit for the pixel size included.
+        '''
+
+        if filename[-3:].lower()=='jpg' or filename[-3:]=='png':
+            #Load file as grayscale image
+            self.image = cv.imread(filename, 0)
+
+        if filename[-3:] in ['tiff', 'tif', 'TIF', 'TIFF']:
+            with tifffile.TiffFile(filename) as tif:
+                self.metadata_tags = {}
+                self.image = tif.asarray()
+                for page in tif.pages:
+                    for tag in page.tags:
+                        self.metadata_tags[tag.name] = tag.value
+
+        if 'XResolution' in self.metadata_tags:
+            if self.metadata_tags['XResolution']==self.metadata_tags['YResolution']:
+                try:
+                    self.pixelSize = 1/(self.metadata_tags['XResolution'][0] / self.metadata_tags['XResolution'][1])
+                except:
+                    pass
+
+        if 'unit' in self.metadata_tags:
+            if self.metadata_tags['unit']=='micron':
+                self.pixelUnit = 'µm'
+            elif self.metadata_tags['unit']=='nm':
+                self.pixelUnit='nm'
+
+        self.filename=filename
+        self.reset_xy()
+
+        if pixelSize:
+            self.pixelSize=pixelSize
+        if pixelUnit:
+            self.pixelUnit=pixelUnit
+
+
+    def open_array(self, array, pixelSize=None, pixelUnit=None, name='Default_image_name'):
+        '''
+        Opens np array into micrograph object. 
+        
+        Parameters
+        ----------
+
+            array:  numpy array
+                The image data in the form of a numpy array 
+
+            pixelSize: float
+                The pixel size in the image, not necessary but can be included here, can also be loaded with set_scale()
+            pixelUnit: str
+                The unit for the pixel size included.
+            name: str
+                The name for the image (becomes micrograph.filename attribute)     
+        '''
+
+        self.image = array 
+        self.reset_xy()
+        self.filename = name
+        if pixelSize:
+            self.pixelSize=pixelSize
+        if pixelUnit:
+            self.pixelUnit=pixelUnit
+
+
+
+
     def write_image(self, name=None, ftype='jpg',outdir=None):
         '''
         Saves the image in a .jpg or .tif file for display or use with other programs. 
@@ -196,7 +340,7 @@ class Micrograph:
                 name = '.'.join(self.filename.split('.')[:-1])
                 #print('else_name = ',name)
         try:
-            name += '_'+self.scalebar_size+'scale.{}'.format(ftype)
+            name += '_'+self.scalebar_size.replace('µ','u')+'scale.{}'.format(ftype)
         except AttributeError:
             name+='.'+ftype
         #if self.foldername!='':
@@ -206,14 +350,20 @@ class Micrograph:
         #else:
         #    newname = self.foldername.strip('\n') + '/' +self.filename.split('.dm')[0]+'_'+self.text+'scale.jpg'
         
+        if self.pixelUnit=='µm':
+            pixelUnit='um'
+        else:
+            pixelUnit=self.pixelUnit
+
         if ftype=='jpg':
             if self.image.max()!=255 or self.image.min()!=0:
                 print(self.image.max(), self.image.min())
                 self = self.convert_to_8bit()
                 print('converting to 8bit')
             cv.imwrite(name,self.image)
+
         elif ftype=='tif':
-            tifffile.imsave(name, self.image,imagej=True, resolution=(1/self.pixelSize, 1/self.pixelSize), metadata={'unit':self.pixelUnit})
+            tifffile.imsave(name, self.image,imagej=True, resolution=(1/self.pixelSize, 1/self.pixelSize), metadata={'unit':pixelUnit})
 
         #self.pil_image.save(name, quality=self.quality)
         print(name, 'Done!')
@@ -419,7 +569,7 @@ class Micrograph:
         return enhanced_object
 
 
-    def Local_normalisation(self,numpatches, padding=0, pad=False):
+    def Local_normalisation(self,numpatches, padding=15, pad=False):
         """
         Normalises contrast across the image, ensuring even contrast throughout. 
         Image is taken and split into patches, following which the median of the patches ('local median') is compared to the median of the image ('global median').
@@ -427,7 +577,7 @@ class Micrograph:
 
         The patch median was chosen so there would be less effect if there is a large dark/bright particle, however if this is too large compared with the patch size it will affect the resulting contrast. 
         
-        Padding can be applied which will greatly reduce edge artefacts. However doing this well will greatly reduce the 
+        Padding can be applied which will greatly reduce edge artefacts. However doing this well will greatly increase the run time.
         Note: if there is a black region (eg. edge of beam or grid) in the image this may lead to bad results.
 
         Parameters
@@ -452,6 +602,8 @@ class Micrograph:
         new_im = deepcopy(self)
         if pad:
             arrs = []
+        else:
+            padding=0
         xconst = int(new_im.image.shape[0]/numpatches)
         yconst = int(new_im.image.shape[1]/numpatches)
         x_coords = [i*xconst for i in range(numpatches)]
@@ -461,9 +613,9 @@ class Micrograph:
         for coord in patch_coords:
             
             x_low = coord[0]
-            x_high = coord[0]+xconst+int(padding*xconst)
+            x_high = coord[0]+xconst+int((padding/100)*xconst)
             y_low = coord[1]
-            y_high = coord[1]+yconst+int(padding*yconst)
+            y_high = coord[1]+yconst+int((padding/100)*yconst)
             
             #local_mean = new_im[coord[0]-xconst:int(coord[0]+xconst*padding), coord[1]-yconst:int(coord[1]+yconst*padding)].mean()
             #.mean()
@@ -503,6 +655,66 @@ class Micrograph:
 
 
     '''
+    def change_scale_unit(self, new_unit, scaling_factor=None):
+        '''
+        Function to change the unit of the scale in the image. The method is built to allow easy transfer between nanometers ('nm') and microns ('µm'), however can be used to give any new unit providing a scaling factor to multiply the current value by is included. 
+
+        Parameters
+        ----------
+
+            new_unit : str
+                The new unit for the scale, commonly 'nm' or 'µm'
+
+            scaling_factor: int/float
+                The value to multiply the current value by to give the new scale. This isnt necessary with 'µm'/'nm' transitions, but without it any other change will change unit only.
+
+        '''
+        if new_unit==self.pixelUnit:
+
+            print('The unit is already {}! Skipping file.'.format(new_unit))
+        elif new_unit=='nm' and self.pixelUnit=='µm':
+            self.pixelSize*=1000
+            self.pixelUnit='nm'
+        elif new_unit!='nm' and new_unit!='µm':
+            self.pixelUnit=new_unit
+
+            if scaling_factor !=None:
+                self.pixelSize*=scaling_factor
+            else:
+                print('Setting new scale unit but not changing the value, please include a scaling factor to also change the value.')
+        elif new_unit=='µm' and self.pixelUnit=='nm':
+            self.pixelSize= self.pixelSize/1000
+            self.pixelUnit='µm'
+
+        elif self.pixelUnit not in ['µm', 'nm'] and new_unit in ['µm','nm'] and scaling_factor==None:
+            print("Error! You want to switch to {} but the transition from {} is not naturally supported, please include a scaling factor (even if its just 1) to ensure correct conversion.".format(new_unit,self.pixelUnit))
+        elif self.pixelUnit not in ['µm', 'nm'] and new_unit in ['µm','nm'] and scaling_factor!=None:
+            self.pixelSize*=scaling_factor
+            self.pixelUnit=new_unit
+
+        else:
+            print('Not sure how we got here! Check inputs and try again - if genuine error, raise an issue on github!')
+
+
+    def set_scale(self, pixels, dist, unit):
+        '''
+        Set the scale in the image with a measurement (number of pixels and size, with unit)
+        
+        Parameters
+        ----------
+        
+            pixels : int/float
+                Number of pixels measured
+
+            dist : int/float
+                Distance measured
+            unit : str
+                Unit of measurement
+
+        '''
+        self.pixelSize=dist/pixels
+        self.pixelUnit=unit
+
     def choose_scalebar_size(self):
         '''
         Function for choosing scalebar size, called through make_scalebar(), not a standalone function.
@@ -529,7 +741,7 @@ class Micrograph:
         #print(x,scalebar_x)
         #possible scalebar sizes are given here, if its >500nm it should be in unit micron, hopefully this should only fail with very extreme examples
         
-        possible_sizes = [0.5, 1,2,5,10,25,50,100,250,500]
+        possible_sizes = [0.5, 1,2,5,10,25,50,100,250,500,1000]
         
         #to select sizes, iterate through possible sizes, if the width of the resulting scalebar (n*pixelsize) 
         #is over 15% of the image size, the size is chose, if none are over 15% of image size
@@ -801,7 +1013,7 @@ class Micrograph:
     used for plotting images using matplotlib. Functions very basic so only meant for rapid use, for better figures write own function or use save command
 
     '''
-    def imshow(self, title=''):
+    def imshow(self, title='', vmax=None, vmin=None):
         '''
         Basic function for plotting the micrograph image
 
@@ -810,12 +1022,24 @@ class Micrograph:
 
             title:str
                 Optional title to be added to the plot 
-
+            
+            vmax: int/float
+                The 'white value' when plotting the image, such that anything above this value is set to white.
+            
+            vmin: int/float
+                The 'black value' when plotting the image, such that anything below this value is set to black.
         '''
-        plt.subplots(figsize=(30,20))
-        plt.imshow(self.image)
+        plt.subplots(figsize=(20,10))
+
+        if not vmax:
+            vmax = self.image.max()
+        if not vmin:
+            vmin = self.image.min()
+
+
+        plt.imshow(self.image, vmax=vmax, vmin=vmin)
         if title!='':
-            plt.title(title, fontsize=30)
+            plt.title(title, fontsize=20)
         plt.show()
 
     def show_pair(self,other_image, title1='', title2=''):
@@ -832,37 +1056,128 @@ class Micrograph:
                 Title for the second image - Optional 
 
         '''
+
         if str(type(other_image))=="<class 'SimpliPyTEM.Micrograph_class.Micrograph'>":
             other_image = other_image.image
-        fig, ax = plt.subplots(1,2, figsize=(30,20))
+        fig, ax = plt.subplots(1,2, figsize=(20,10))
         ax[0].imshow(self.image)
         if title1!='':
-            ax[0].set_title(title1, fontsize=30)
+            ax[0].set_title(title1, fontsize=20)
 
         ax[1].imshow(other_image)
         if title2!='':
-            ax[1].set_title(title2, fontsize=30)
+            ax[1].set_title(title2, fontsize=20)
         plt.show()
 
-    def plot_histogram(self, sidebyside=False):
+    def plot_histogram(self, sidebyside=False, vmax=None, vmin=None):
+        '''
+        Function to plot a histogram of the image values. This can be done on its own or side by side with the image.
+        
+        Parameters
+        ----------
+            sidebyside:bool
+                Show the image next to the histogram? True or False (default is False)
+
+            vmax: int/float
+                The 'white value' when plotting the image, such that anything above this value is set to white.
+            
+            vmin: int/float
+                The 'black value' when plotting the image, such that anything below this value is set to black.
+
+    
+        '''
         if sidebyside:
-            fig, ax = plt.subplots(1,2, figsize=(30,10))
-            ax[0].imshow(self.image)
+
+            if not vmax:
+                vmax = self.image.max()
+            if not vmin:
+                vmin = self.image.min()
+
+            fig, ax = plt.subplots(1,2, figsize=(20,10))
+            ax[0].imshow(self.image, vmax=vmax, vmin=vmin)
+            ax[0].tick_params(axis='x', labelsize=20)
+            ax[0].tick_params(axis='y', labelsize=20)
             if self.image.dtype == 'unit8':
                 ax[1].hist(self.image.ravel(), 256, [0,256])
             else:
                 ax[1].hist(self.image.ravel(), 100)
             ax[1].set_xlabel('Pixel Values', fontsize=20)
             ax[1].set_ylabel('Frequency',fontsize=20)
+            ax[1].tick_params(axis='x', labelsize=20)
+            ax[1].tick_params(axis='y', labelsize=20)
         else:
             plt.figure(figsize=(8,5))
             if self.image.dtype == 'unit8':
                 plt.hist(self.image.ravel(), 256, [0,256])
             else:
                 plt.hist(self.image.ravel(), 100)
-            plt.xlabel('Pixel Values', fontsize=20)
-            plt.ylabel('Frequency',fontsize=20)
+            plt.xlabel('Pixel Values', fontsize=15)
+            plt.ylabel('Frequency',fontsize=15)
+
         plt.show()
+
+
+    def display_fft(self,sidebyside=False, vmax=None,vmin=None, ret=False):
+        '''
+        Function to display or return the power-spectrum, or 2D fourier transform of the image. This can be useful to observe any periodic features, eg lattace lines, or Thon rings resulting from the contrast transfer function in the microscope.
+
+
+        Parameters
+        ----------
+
+            sidebyside:bool 
+                Do you want to display the power spectrum next to the image? default is yes (True), change to False to see fft only
+
+            vmax: int/float
+                The 'white value' when plotting the power spectrum, such that anything above this value is set to white.
+            
+            vmin: int/float
+                The 'black value' when plotting the power spectrum, such that anything below this value is set to black.
+            
+            ret: bool 
+                Set to true if you want to return a copy of the fft as a micrograph object (with the fft as the .image parameter)
+
+        Returns
+        -------
+
+            If ret==True, a new micrograph object is returned with the power spectrum as the image, from here the same functions can be used 
+
+        '''
+        fft = np.fft.fft2(self.image)
+        fshift = np.fft.fftshift(fft)
+        magnitude_spectrum = np.log(np.abs(fshift))
+        magnitude_spectrum = magnitude_spectrum.astype(np.float32)
+
+
+        if not vmax:
+            vmax = magnitude_spectrum.max()
+        if not vmin:
+            vmin = magnitude_spectrum.min()
+        #print(magnitude_spectrum.dtype)
+
+        if ret==True: 
+
+            fft_ob = Micrograph()
+            fft_ob.image = magnitude_spectrum
+            filename = self.filename.split('.')[:-1]
+            filename.append('_FFT')
+            fft_ob.filename = filename
+            return fft_ob
+
+        if sidebyside:
+            fig,ax= plt.subplots(1,2, figsize=(30,20))
+            ax[0].imshow(self.image)
+            ax[0].set_title('Image')
+            ax[1].imshow(magnitude_spectrum, vmin=vmin, vmax=vmax)
+            ax[1].set_title('Power spectrum')
+            plt.show()
+        else:
+            plt.figure(figsize=(20,20))
+            plt.imshow(magnitude_spectrum, vmin=vmin, vmax=vmax)
+            plt.show()
+
+
+
     '''--------------------------------------------------------------------------------
     SECTION: METADATA
 
