@@ -5,9 +5,9 @@ from PyQt5.QtWidgets import (QApplication, QLabel, QPushButton, QVBoxLayout, QWi
                              QFileDialog, QTextEdit, QTextBrowser, QComboBox, QCheckBox, QMainWindow,QGridLayout,QFileDialog,QLineEdit,QButtonGroup)
 from PyQt5.QtGui import QFont
 import sys
-from App_functions import * 
+from SimpliPyTEM.App_functions import * 
 from SimpliPyTEM.PDF_generator import *
-from html_writer import *
+from SimpliPyTEM.html_writer import *
 import threading 
 
 
@@ -17,7 +17,7 @@ class MainApplication(QWidget):
         #Qt.AlignHCenter
 
         self.setGeometry(500, 500, 100,400)
-        self.setWindowTitle('Micrograph Previews App')
+        self.setWindowTitle('SimpliPyTEM-GUI')
         
         self.title = QLabel('Make Preview Images',self)
         #app.setStyleSheet("QLabel, QCheckBox{font-size: 16pt;}")
@@ -308,7 +308,7 @@ class MainApplication(QWidget):
             print('The Images folder was not found, please make sure that the output folder defined above has a folder of images called images.')
             os.chdir(cwd)
             return 1
-        print(images)
+        print(image_files)
 
         write_html(image_files, video_files, title, notes)
         write_css()
